#include <SoftwareSerial.h>
#include <Servo.h>
#include <wiring_private.h>
#include <pins_arduino.h>

/***   Global variables   ***/
SoftwareSerial btSerial((2),(4));
Servo _servo9;
Servo _servo5;
int _white=40;
int _black=400;
const uint16_t CHAMPIONS[] = {698,750,659,187,698,187,659,375,523,562,440,187,587,375,440,750,0,562,0,375,523,187,698,750,784,187,880,187,1047,375,880,562,587,187,659,187,587,750};

/***   Function declaration   ***/
void move_base_9_5(int speed, int advance, int turn);
int follow(int light, int gain);
unsigned long _pulseIn(uint8_t pin, uint8_t state, unsigned long timeout);
long US_init(int trigger_pin, int echo_pin);
long distance(int trigger_pin, int echo_pin);
void evita();
void sigue();
void LEDMatrix_init(int cs, int din, int clk);
void writeRow(int cs, int din, int clk, int row, int data);
void maxAll (int cs, int din, int clk, int reg, int col);
void putByte (int din, int clk, int data);
void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8);
void playMelody(int pin,const uint16_t* melody, int length);

/***   Tasks declaration   ***/

/***   ISR function declaration   ***/

void setup()
{
  btSerial.begin(9600);
  Serial.begin(19200);

  pinMode( (7) , INPUT );
  pinMode( (13) , OUTPUT );
  pinMode(10,OUTPUT);
  pinMode(12,OUTPUT);
  pinMode(11,OUTPUT);
  pinMode(8,OUTPUT);

  _servo5.attach(5);
  _servo9.attach(9);
  move_base_9_5(0,0,0);
  

  maxAll(10,12,11,11,7);
  maxAll(10,12,11,9,0);
  maxAll(10,12,11,12,1);
  maxAll(10,12,11,15,0);
  int i=0;
  for (i = 1; i <= 8; i++)
    maxAll(10,12,11,i,0);
  maxAll(10,12,11,10,15);
}


void loop()
{
      int cmd;
      if (btSerial.available()>0){
      cmd=btSerial.read();
      }
      switch(cmd){
        case 42:
        sigue();
        break;

        case 41:
        evita();
        delay(20);
        break;

        case 21:
        move_base_9_5(50,50,8);
        delay(20);
        break;

        case 22:
        move_base_9_5(50,(-50),8);
        delay(20);
        break;

        case 23:
        move_base_9_5(20,20,-100);
        delay(20);
        break;

        case 24:
        move_base_9_5(20,20,100);
        delay(20);
        break;

        case 25:
        move_base_9_5(0,0,0);
        delay(20);
        break;

        case 1:
        expression(10,12,11,195,230,102,12,12,102,230,195);
        delay(20);
        break;

        case 2:
        expression(10,12,11,12,246,246,3,3,246,246,12);
        delay(20);
        break;

        case 3:
        expression(10,12,11,3,246,246,12,12,246,246,3);
        delay(20);
        break;

        case 4:
        expression(10,12,11,48,120,124,62,62,124,120,48);
        delay(20);
        break;

        case 5:
        expression(10,12,11,0,224,239,15,15,239,224,0);
        delay(20);
        break;

        case 11:
        expression(10,12,11,0,0,0,0,0,0,0,0);
        delay(20);
        break;

        case 17:
        playMelody(8,CHAMPIONS,sizeof(CHAMPIONS)/(2*sizeof(uint16_t)));
        delay(20);
        break;
      }
}

/***   Function definition   ***/
void sigue()
{
  move_base_9_5(50,35,follow(analogRead((A0)),12));
  delay(20);
}

void evita()
{
  int dist=distance((13),(7));
  int ref=12;
  delay(20);
  if (dist>ref){
  move_base_9_5(50,30,8);
  delay(20);  
  }
  if (dist<=ref){
  move_base_9_5(0,0,0);
  delay(20);
  move_base_9_5(50,30,90);
  delay(400);
  move_base_9_5(0,0,0);
  delay(400);
  dist=distance((13),(7));
  delay(20);
  if (dist>ref){
  move_base_9_5(0,0,0);
  delay(100);
  }
  if (dist<=ref){
  move_base_9_5(50,30,-90);
  delay(700);
  move_base_9_5(0,0,0);
  delay(400);
  }
 }
}

void move_base_9_5(int speed, int advance, int turn)
{
  _servo9.attach(9);
  _servo5.attach(5);
_servo9.write(90+(int)(65.0*(float)(speed*(advance-turn))/10000.0));
_servo5.write(90-(int)(65.0*(float)(speed*(advance+turn))/10000.0));
}

int follow(int light, int gain)
{
  return (int)(((float)(gain)/100.0)*((float)((_white-_black)/2+_black)-(float)(light)));
}

unsigned long _pulseIn(uint8_t pin, uint8_t state, unsigned long timeout)
{
  uint8_t bit = digitalPinToBitMask(pin);
  uint8_t port = digitalPinToPort(pin);
  uint8_t stateMask = (state ? bit : 0);
  unsigned long width = 0;
  unsigned long numloops = 0;
  unsigned long maxloops = microsecondsToClockCycles(timeout) / 16;
  while ((*portInputRegister(port) & bit) == stateMask)
    if (numloops++ == maxloops)
      return 0;
  while ((*portInputRegister(port) & bit) != stateMask)
    if (numloops++ == maxloops)
      return 0;
  while ((*portInputRegister(port) & bit) == stateMask) {
    if (numloops++ == maxloops)
      return 0;
    width++;
  }
  return clockCyclesToMicroseconds(width * 21 + 16);
}
long US_init(int trigger_pin, int echo_pin)
{
  digitalWrite(trigger_pin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigger_pin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigger_pin, LOW);
  long microseconds = _pulseIn(echo_pin ,HIGH,100000);
  return microseconds;
}
long distance(int trigger_pin, int echo_pin)
{
  long microseconds = US_init(trigger_pin, echo_pin);
  long distance;
  distance = microseconds/29/2;
  if (distance == 0){
    distance = 999;
  }
  return distance;
}
void LEDMatrix_init(int cs, int din, int clk) {
maxAll(cs,din,clk,11,7);
maxAll(cs,din,clk,9,0);
maxAll(cs,din,clk,12,1);
maxAll(cs,din,clk,15,0);
int i=0;
for (i = 1; i <= 8; i++) {
maxAll(cs,din,clk,i, 0);
}
maxAll(cs,din,clk,10,15);
}
void writeRow(int cs, int din, int clk, int row, int data) {
digitalWrite(cs,LOW);
putByte(din,clk,row);
putByte(din,clk,data);
digitalWrite(cs,LOW);
digitalWrite(cs,HIGH);
}
void maxAll (int cs, int din, int clk, int reg, int col) {
digitalWrite(cs,LOW);
putByte(din,clk,reg);
putByte(din,clk,col);
digitalWrite(cs,LOW);
digitalWrite(cs,HIGH);
}
void putByte (int din, int clk, int data) {
byte i = 8;
byte mask;
while(i > 0) {mask = 0x01 << (i - 1);
    digitalWrite(clk,LOW);
    if (data & mask)
{
      digitalWrite(din,HIGH);
    }else{
      digitalWrite(din,LOW);
    }
    digitalWrite(clk,HIGH);
    --i;
}
}
void expression(int cs, int din, int clk, int col1, int col2, int col3, int col4, int col5, int col6, int col7, int col8) {
writeRow(cs,din,clk,1, col1);
writeRow(cs,din,clk,2, col2);
writeRow(cs,din,clk,3, col3);
writeRow(cs,din,clk,4, col4);
writeRow(cs,din,clk,5, col5);
writeRow(cs,din,clk,6, col6);
writeRow(cs,din,clk,7, col7);
writeRow(cs,din,clk,8, col8);
}

void playMelody(int pin,const uint16_t* melody, int length)
{
  unsigned int note;
  unsigned long duration;
  uint16_t* melody_ptr=(uint16_t*)melody;
  for (int i=0;i<length;i++)
  {
    note=*melody_ptr++;
    duration=*melody_ptr++;
    tone(pin,note,duration);
    delay(duration);
    noTone(pin);
  }
}
